#!/bin/bash

###
#  Data download and creation script for Apache Derby
#  Copyright (C) 2015 Carol Alexandru
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
###

repo="https://github.com/google/guava.git"

jardownloads=(
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/10.0.1/guava-10.0.1.jar"
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/11.0.2/guava-11.0.2.jar"
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/12.0.1/guava-12.0.1.jar"
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/13.0.1/guava-13.0.1.jar"
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/14.0.1/guava-14.0.1.jar"
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/15.0/guava-15.0.jar"
  "http://search.maven.org/remotecontent?filepath=com/google/guava/guava/16.0.1/guava-16.0.1.jar"
)

versions=( \
  "10.0.1" \
  "11.0.2" \
  "12.0.1" \
  "13.0.1" \
  "14.0.1" \
  "15.0" \
  "16.0.1" \
)

steps=(
  "10.0.1-11.0.2" \
  "11.0.2-12.0.1" \
  "12.0.1-13.0.1" \
  "13.0.1-14.0.1" \
  "14.0.1-15.0" \
  "15.0-16.0.1" \
)


declare -A commits=( \
  ["10.0.1"]="22eb4ada1211093d21b6a209f8e03d107ba48580" \
  ["11.0.2"]="78c34456423e680ec69059f21a5841296ab23277" \
  ["12.0.1"]="585f008152ce3961ef33c9928121dc9344fc1118" \
  ["13.0.1"]="c523556ab7d0f05afadebd20e7768d4c16af8771" \
  ["14.0.1"]="c3bd8eb49f9b355a140cbb56ab592e973fea4c0d" \
  ["15.0"]="7ce15119ae3b8ea71bdfd43bd6a0b96f851187d0" \
  ["16.0.1"]="0ba7ccf36f5384a321cb78d62375bf7574e7bc24" \
)

if true; then
  rm -rf jars
  mkdir jars
  cd jars
  echo "Downloading jar files..."
  echo "${jardownloads[@]}" | xargs -n 1 -P 8 wget -q
  cd ..
fi

repodir="gitrepo"
if [[ ! -e "$repodir" ]]; then
  mkdir "$repodir"
  git clone "$repo" "$repodir"
fi


function getJars {
  find "jars" -name "*$1*.jar"
}

## compute ck metrics
if true; then
  rm -Rf ckjm-results
  mkdir -p ckjm-results/{raw,aggregated}
  for i in "${versions[@]}"; do
    echo "computing ckjm metrics for version $i"
    jars=$(getJars $i)
    raw="ckjm-results/raw/""$i"".ssv"
    aggregated="ckjm-results/aggregated/""$i"".ssv"
    withFileNames="ckjm-results/withFileNames/""$i"".ssv"

    for jar in $jars; do
      echo $jar
      jar tf "$jar" | grep -E "\.class$" | awk '{print "'"$jar"' " $1}' | java -jar ../ckjm-1.9.jar 2>/dev/null 1>>"$raw"
    done

    cat "$raw" | sort | \
      awk '{n=NF;sub(/\$.*/,"",$1);d[$1];for (i=2;i<=n;i+=1) cols[$1 i]+=$i}END{for (e in d) { printf "%s",e;for (i=2;i<=n;i+=1) printf " %s",cols[e i];printf "\n"}}' | \
      grep -E '^com\.google' | sort > "$aggregated"
  done
fi

## compute line diffs
if true; then
  rm -Rf diff-results
  mkdir -p diff-results/{raw,filtered}
  cd "$repodir"
  for i in "${steps[@]}"; do
    echo "computing line diffs between $i"
    first=(${i%%-*})
    second=(${i##*-})
    firstRef="${commits[$first]}"
    secondRef="${commits[$second]}"
    raw="../diff-results/raw/""$i"".tsv"
    filtered="../diff-results/filtered/""$i"".tsv"

    git diff --numstat "$firstRef" "$secondRef" | grep -E '.java$' > "$raw"


    cat "$raw" | sed -e 's/\([0-9]*\t[0-9]*\t\).*com\/google/\1com\/google/g;s/\.java//g;s/\//\./g' | awk '{print $3 " " $1 " " $2}' | grep -E '^com\.google' > "$filtered"
  done
  cd ..
fi

# compute sloc
if true; then
  rm -Rf sloc-results
  mkdir -p sloc-results/{raw,filtered}
  cd "$repodir"
  for i in "${versions[@]}"; do
    echo "counting lines of code for $i"
    raw="../sloc-results/raw/""$i"".ssv"
    filtered="../sloc-results/filtered/""$i"".ssv"
    ref="${commits[$i]}"
    git checkout "$ref"
    cloc --csv-delimiter=' ' --quiet --by-file ./ 2>&1 | grep -e '^Java \.\/guava\/' | sort -k2 | sed -e 's/^Java \.\///g' > "$raw"
    cat "$raw" | sed -e 's/.*com\/google/com\/google/g;s/\.java//g;s/\//\./g' | grep -E '^com\.google' > "$filtered"
  done
  cd ..
fi

# join the metrics and line diff results
if true; then
  rm -Rf synthesized-results
  mkdir -p synthesized-results/{join1,clean1,join2,final}
  for i in "${steps[@]}"; do
    first=(${i%%-*})
    echo "synthesizing results using metrics from version $first and line diffs between $i"
    metrics="ckjm-results/aggregated/""$first"".ssv"
    slocs="sloc-results/filtered/""$first"".ssv"
    diffs="diff-results/filtered/""$i"".tsv"
    join1="synthesized-results/join1/""$i"".ssv"
    join2="synthesized-results/join2/""$i"".ssv"
    clean1="synthesized-results/clean1/""$i"".ssv"
    final="synthesized-results/final/""$i"".csv"

    join -a 1 -1 1 -2 1 -e 0 <(sort -k1b,1 "$metrics") <(sort -k1b,1 "$diffs") > "$join1"

    cat "$join1" | awk '{if (NF==9) { $10="0"; $11="0"; }; print $0}' > "$clean1"

    join -a 1 -1 1 -2 1 -e 0 <(cat "$clean1" | sort -k1b,1) <(cat "$slocs" | sort -k1b,1) > "$join2"

    echo "fileclass,wmc,dit,noc,cbo,rfc,lcom,ca,npm,addedlines,removedlines,blanklines,commentlines,codelines" > "$final"
    cat "$join2" | sed -e 's/ /,/g' >> "$final"
  done
fi

